--------------------------------------------------------------------------------
-- Title				:	RAM(16X8)
-- Project			:	Sample code
-- File				:	ram16X8.vhd
-------------------------------------------------------------------------------
-- Organization	:	ni logic Pvt. Ltd., Pune, India
-------------------------------------------------------------------------------
-- Description		:	Implementing 16x8 RAM in VHDL
--						USDP has Active 'LOW' Reset
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity ram16X8 is
port 
		( 
			wr,rd		: 	in std_logic;		-- write & read i/ps, active LOW
			cs			: 	in std_logic;		-- chip select, active LOW
         data 		: 	inout std_logic_vector(7 downto 0); -- data port
			add 		: 	in integer range 0 to 15	    		-- 4-bit address i/p
		);
end ram16X8;

architecture Behavioral of ram16X8 is
type ram_type is array (15 downto 0) of std_logic_vector (7 downto 0); 
signal ram : ram_type;

begin

-- writing in RAM, read should be disabled and data should be written by external
-- device on the data port.	
process(wr,data)
begin
	if (wr'event and wr='1')then
		if (cs = '0') then 
			ram(add) <= data; 
		end if;
	end if; 
end process; 


-- reading from RAM
data	<=	ram(add) when rd='0' and cs='0' else
			(others=>'Z');

end Behavioral;